#ifndef TMY_LIST_H
#define TMY_LIST_H
#include <iostream.h>
#include <iomanip.h>


//
//
// 

#include "TrecordMemoryPool.h"

//
// class: TmyList
//
// 		simple link list of integers
//
class TmyList{

	//
	// Gives TmyListIterator full access to this
	// list. TmyListIterator will be responsible
	// for traversing this list for non-exclusive accesses
	// (e.g. print/search).
	//
	friend class TmyListIterator;

	private:
		TrecordMemoryPool&	 fRecordPool;		// Memory manager for this list

		TrecordMemoryAddress fHeadPointer;		// head of the list

		int					 fNumOps;
								//
								// Remembers how many times the user have performed
								// operations on this link list.
								//
								// Each Operation is either an insert or delete
								// Unsuccessul deletion is also recorded as a valid op
								//

	public:
		//
		// this is the constructor
		// I am initializing instance 
		// variables in my constructor
		//
		TmyList(TrecordMemoryPool &pool) :
				fNumOps(0),
				fRecordPool(pool),						// initialize record pool
				fHeadPointer(kNullAddress)	 			// initialize HeadPointer to NULL
			{}		


		TrecordMemoryAddress HeadOfList() const {
				return fHeadPointer;
		}
				
		//
		//
		// simple list insertion/deletion methods
		//
		void 		InsertElement(int Key);
					// insert Key into the link list
					// a new link will be allocated (by requesting
					// for a new one from the TrecordMemoryPool)
					// We must check for out-of-memory sitation and
					// Warn the user accordingly.
					// 
					// Your insetion must be O(1) (must be 
					// independent of the length of the list)
					
		bool		RemoveLinkFromList(int Key);    
					// remove the link entry containing Key
					// no memory is delete/free/return should be
					// attempted. Our TrecordMemoryPool will 
					// perform garbage collection when necessary


		int			NumOpsPerformed() const { return fNumOps; }
		//
		//
		//
		// method that prints out the content of the link list.
		// together with the content of the TrecordMemoryPool
		//
		void		PrintListContent(const char* listName);
					// print the keys and the next field in the list

		void		SortList();
		
};

class TmyListIterator {
	private:
		const TrecordMemoryPool		&fRecordPool;	// Record pool of list
		TrecordMemoryAddress 		fEntry;
	
	public:

		TmyListIterator(const TmyList &list) :
				fEntry(list.fHeadPointer), fRecordPool(list.fRecordPool) 
			{}

		
		void IterateToNext() { fEntry = fRecordPool.NextField(fEntry); }
		bool EndOfList() const { return fEntry == kNullAddress; }
		bool NotEndOfList() const { return !EndOfList(); }

		TrecordMemoryAddress GetLink() const { return fEntry; }
		int	 GetKey() const { return fRecordPool.KeyField(fEntry); }
};

#endif
