#include "TrecordMemoryPool.h"

TrecordMemoryPool::TrecordMemoryPool() 
{
	fCurrentRecord = 0;
	SetAllRecordsToFree();
}


bool TrecordMemoryPool::AtEndOfPool() 
{
	return (fCurrentRecord > NUM_RECORDS_IN_POOL );
}


void TrecordMemoryPool::GarbageCollection(TrecordMemoryAddress headPtr) 
{

	SetAllRecordsToFree();

	TrecordMemoryAddress addr = headPtr;
	while ( addr != kNullAddress ) {
		SetRecordToInUse(addr);
		addr = NextField(addr);
	}
	//
	// Start from the beginning again
	//
	fCurrentRecord = 0;
}

void TrecordMemoryPool::SetNextField(TrecordMemoryAddress ptr, TrecordMemoryAddress next)  
{
	fRecordMemory[ptr+kNextFieldOffset] = next;
}

void TrecordMemoryPool::SetKeyField(TrecordMemoryAddress ptr, int key)
{
	fRecordMemory[ptr+kKeyFieldOffset]	= key;
	SetRecordToInUse(ptr);
}


TrecordMemoryAddress 
TrecordMemoryPool::FindNextFreeRecord()
{
	TrecordMemoryAddress result = kNullAddress;
	TrecordMemoryAddress ptr = kNullAddress;
	bool found = false;
	
	while ( (!AtEndOfPool()) && (!found) ) {
		ptr = RecordAddress(fCurrentRecord);
		if (RecordIsFree(ptr)) {
			// found it!
			SetRecordToInUse(ptr);
			result = ptr;
			found = true;
		} 
		fCurrentRecord++;
	}
	return result;
}

TrecordMemoryAddress 
TrecordMemoryPool::NextFreeRecord(TrecordMemoryAddress headPtr) 
{
	TrecordMemoryAddress ptr = FindNextFreeRecord();

	if (ptr == kNullAddress) {
		//
		// We have came to the end of Pool and not found a free one.
		// GarbageCollection is needed.
		GarbageCollection(headPtr);

		ptr = FindNextFreeRecord();
	}

	return ptr;
}

void
TrecordMemoryPool::PrintRecordMemoryPool()
{
	cout << "Record Memory Pool Content: " << endl;
	for (int i=0; i<NUM_RECORDS_IN_POOL; i++) {
		TrecordMemoryAddress ptr = RecordAddress(i);

		cout << "Memory: ["  << setw(2) << ptr
		     << "]: Record(" << setw(2) << i 
			 << "): Key="    << setw(5) << KeyField(ptr)
			 << "  Next="    << setw(5) << NextField(ptr)
			 << "  MM="      << (MMField(ptr)==eRecordInUse ? "InUse" : "Free") << endl;
	}
}
