#ifndef TRECORD_MEMORY
#define TRECORD_MEMORY
#include <iostream.h>
#include <iomanip.h>

	//
	// number of records
	// size of each record
	// and the total memory size
	//
#define NUM_RECORDS_IN_POOL 5
#define RECORD_SIZE			3
#define RECORD_POOL_SIZE    NUM_RECORDS_IN_POOL*RECORD_SIZE

#define kNullAddress		-1

typedef int TrecordMemoryAddress;

//
// Pool of record memory: 
//
// 	Design to support one link list of integerRecords.
// 	A IntergerRecord is defined as:
//
// 		Offset - 0: Key
// 		Offset - 1: Next "Pointer to next integerRecord"
// 		Offset - 2: Reserved for Memory managenent
//  
// 
class TrecordMemoryPool {
	private:
		enum {
			eRecordInUse  = -1,
			eRecordIsFree = -2
		};

		#define kKeyFieldOffset  0
		#define kNextFieldOffset 1
		#define kMMFieldOffset   2

		int	fRecordMemory[RECORD_POOL_SIZE];

		int fCurrentRecord;	
				// this is the nth record, this must be converted
				// to RecordAddress() before we can use to reference
				// into the fRecordMemory[]

		TrecordMemoryAddress RecordAddress(int nthRecord) {
				return nthRecord * RECORD_SIZE;
		}

		int  MMField(TrecordMemoryAddress ptr) { return fRecordMemory[ptr+kMMFieldOffset]; }

		void SetRecordToInUse(TrecordMemoryAddress ptr) {
				fRecordMemory[ptr+kMMFieldOffset] = eRecordInUse;
		}

		void SetRecordToFree(TrecordMemoryAddress ptr) {
				fRecordMemory[ptr+kMMFieldOffset] = eRecordIsFree;
		}

		bool RecordIsFree(TrecordMemoryAddress ptr) {
					return (fRecordMemory[ptr+kMMFieldOffset] == eRecordIsFree);
		}

		void SetAllRecordsToFree() {
			for (int i=0; i<NUM_RECORDS_IN_POOL; i++)
				SetRecordToFree( RecordAddress(i) );
		}

		TrecordMemoryAddress FindNextFreeRecord();
		//
		//=============================================================================
		//

		bool AtEndOfPool();
			//
			// At the end of pool, garbage collection should start now
			//

	public:
		//
		// Constructor
		TrecordMemoryPool();

		void GarbageCollection(TrecordMemoryAddress headPtr);
			// Initiate garbage collection knowing that headPtr
			// points to a link list defined in this pool

		void SetKeyField(TrecordMemoryAddress ptr, int key);
			// ptr - the address of the record
			// key - integer value to be put into the record
			
		void SetNextField(TrecordMemoryAddress ptr, TrecordMemoryAddress nextPtr);
			// ptr - the address of the record
			// nextPtr - pointer to be put into the next field
			
		int KeyField(TrecordMemoryAddress ptr) const { return fRecordMemory[ptr]; }
			// ptr is the address to the beginning of a record
			// function returns the KeyField of this record
			//

		TrecordMemoryAddress NextField(TrecordMemoryAddress ptr) const { return fRecordMemory[ptr+1]; }
			// ptr is the address to the beginning of a record
			// this function returns the address to the next 
			// record (or kNullAddress if there is no next record)

		TrecordMemoryAddress NextFreeRecord(TrecordMemoryAddress headPtr);
			// headPtr - address to the beginning of the list
			// 		this function will return the adress to a free record
			// 		garbage collection will be triggered if necessary
			// 		the "headPtr" is needed for the garbage collection

		void PrintRecordMemoryPool();
			// Dump out the entire content of the memory pool
			// from smaller memory to larger memory
};

#endif
