//////////////////////////////////////////////////
// File: TthreadBase.cpp
// Author: Kelvin Sung
// Date: April 2003
// 
// This example shows how to use _beginthread and _endthread to begine
// new threads. Nothing much happens here. Notice:
//
//    1. Each thread has a unique TthreadBase object 
//			(i.e. different name/id/ofstream)
//
//    2. All threads shares the same global variables with the main thread
//		 	i.e. cerr is the same one!!
//
//
//
#include "TthreadBase.h"

#include <windows.h>
			// for Sleep

#include <iostream.h>
#include <iomanip.h>
			// for cout/cin/cerr, and i/o formatting

#include <fstream.h>
			// for ostream


#include <process.h>
			// for _beginthread and _endthread

#include <stdio.h>
			// for sprintf


// #inlcude <afxmt.h>
//			// for MFC synchronization objects: CMutex/CEvent
//	

//
// To Compile: for Both Debug and Release:
//    Must include compiler optins: /MT /D "_X86_"
//
// To Link: 
//          For Debug Must include:   LIBCMTD.lib and MSVCRTD.lib
//			For Release Must include: LibCMT.LIB and MSVCRT.LIB
//
//    NAFXCWD.lib (NAFXCW.LIB for Release) must appear _BEFORE_ LIBCMTD.LIB)
//
//
//

#define kNumJob 	5

static void StupicThreadWorker(void *parm)
	//
	// The work we do: loop throught kNumJob times
	// ThreadDelay, and then quits!
	//
{
	TthreadBase *td = (TthreadBase *) parm;
	ofstream &os = td->OutStream();
	
	for (int i=0; i<kNumJob; i++) {
		os << "Thread: [" << td->ThreadName() << "]: Working on (" << setw(3) << i << ")" << endl;
		td->ThreadWorkTime();
	}

	cerr << "Thread: [" << td->ThreadName() << "]: Done!" << endl;

	_endthread();

}

///////////////////////////////////////////////////////////////////////////////////////
//
// ThreadBase methods
//
/////////////////////////////////////////////////////////////////////////////////////

TthreadBase::TthreadBase(char *name, int id, int tm)
	//
	// Constructor of ThreadBaes
	//
{
	fID = id;
	fWorkTime = tm * 1000; 	// convert from ms to secs
	sprintf(fName, "%s.%d", name, fID);


	//
	// Create an output pipe for output messages
	//
	fOutStream = new ofstream(fName);

	*fOutStream << "Thread: [" << fName << "] Initialization Completed" << endl;
}

void
TthreadBase::ThreadWorkTime()
{
	Sleep(fWorkTime);
}

void
TthreadBase::StartThread()
{
	_beginthread(StupicThreadWorker, 0, (void *) this );
}

